/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.itis;

import java.io.IOException;
import org.openrdf.model.URI;
import org.openrdf.elmo.Entity;
import it.tidalwave.bluebill.taxonomy.elmo.ElmoTaxonomyVocabulary;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import org.openrdf.model.Literal;
import org.openrdf.elmo.LiteralManager;
import org.openrdf.elmo.ResourceManager;
import org.openrdf.elmo.sesame.SesameManager;
import org.openrdf.model.Resource;
import org.openrdf.model.impl.URIImpl;
import javax.annotation.Nonnull;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxon.Builder;
import it.tidalwave.bluebill.taxonomy.birds.col.CatalogueOfLifeImporter;
import it.tidalwave.util.Initializer;
import it.tidalwave.util.NotFoundException;
import org.openrdf.repository.RepositoryException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ItisImporter extends CatalogueOfLifeImporter
  {
    private static final String CLASS = ItisImporter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final transient ItisCodeMap itisCodeMap = new ItisCodeMap();

    @Override
    protected void initialize()
      throws IOException
      {
        super.initialize();
        itisCodeMap.initialize();
      }

    @Nonnull
    protected Builder process (final @Nonnull String path,
                               final @Nonnull String scientificName,
                               @Nonnull Builder builder)
      throws Exception
      {
        try
          {
            final String itisCode = itisCodeMap.findCodeByPath(path);
            builder = builder.withInitializer(new Initializer<Taxon>()
              {
                @Nonnull
                public Taxon initialize (final @Nonnull Taxon taxon)
                  {
                    try
                      {
                        final SesameManager em = (SesameManager) ElmoManagerThreadLocal.get();
                        final ResourceManager<Resource> rm = em.getResourceManager();
                        final LiteralManager<URI, Literal> lm = em.getLiteralManager();
                        em.getConnection().add(rm.createResource(taxon.getLookup().lookup(Entity.class).getQName()),
                                               new URIImpl(ElmoTaxonomyVocabulary.NS_TAXO_BASE + "itisCode"),
                                               lm.getLiteral(itisCode));
                        return taxon;
                      }
                    catch (RepositoryException e)
                      {
                        throw new RuntimeException(e);
                      }
                  }
              });

            return builder;
          }
        catch (NotFoundException e)
          {
            throw new RuntimeException("No ITIS code for " + path);
          }
      }
  }
